;;; @file c2n232.s
;;; Transfer routines for the Commodore cassette drive emulator C2N232
;;; designed by Marko Mkel
;;; @author Marko Mkel (marko.makela@nic.funet.fi)

	;; flag: is the transfer always in slow mode?
allslow = 1
	;; flag: does the receive routine leave the y register alone?
y_unmod = 0
	;; macro: additional commands for info mode
#mac infoinit
	lda #$f0
	jsr send
#endm

	;; macro: initialize the cable
#mac initcable
#if host == 16
#else
#if host >= 201 && host <= 203 && c2nrdval == $80
	bit c2n1read-1	; clear the cassette #1 read status
#else
#if clearviaifr
	lda #c2nrdval
	sta c2nread	; clear the cassette read status
#else
	bit c2nread	; clear the cassette read status
#endif
#endif
#endif
#endm
	;; macro: deinitialize the cable
#mac deinitcable
#endm

	;; macro: wait for high-to-low transition on CASS READ
#mac c2nrdlo
#if c2nrdval == $80
1$:	bit c2nread
	bpl 1$
#else
	lda #c2nrdval
1$:	bit c2nread	; wait for CASS READ low
#if host == 16
	bne 1$
#else
	beq 1$
#endif
#endif
#endm

	;; macro: clear the status of the CASS READ line
#mac c2nclrd
#if host >= 201 && host <= 203
#if c2nrdval == $80
	bit c2n1read-1
#endif
	;; For PET cassette #2, the interrupt is need not be cleared here,
	;; since accessing c2nwrite has already cleared it.
#else
#if clearviaifr
	sta c2nread
#endif
#endif
#endm

	;; macro: determine whether a request is pending
#mac isrequest
	lda c2nwrite
#if host == 16
	ora #c2nwrval
#else
	and #~c2nwrval
#endif
	sta c2nwrite		; drop the cassette write line
	sta send_nibble + 1
#if host == 16
	and #~c2nwrval
#else
	ora #c2nwrval
#endif
	tax
#if c2nrdval == $80
	bit c2nread
	bmi noret$
#else
	lda #c2nrdval
	bit c2nread		; test if CASS READ is low
#if host == 16
	beq noret$
#else
	bne noret$
#endif
#endif
	stx c2nwrite		; raise the cassette write line
	jmp return
noret$:	c2nclrd
#endm

	;; macro: read a bit to the Carry flag
#mac c2nrdbit
#if host >= 201 && host <= 203 && c2nrdval == $80
	lda c2nread
#else
	lda #c2nrdval
	and c2nread
#endif
	c2nclrd
	cmp #c2nrdval
#endm

	; .A := data, .Y := 00, .X preserved
receive_switch:
receive:
#if host == 16
	lda #c2nwrval
	ora c2nwrite
#else
	lda #~c2nwrval
	and c2nwrite
#endif
	sta c2nwrite	; drop WRITE
	c2nrdlo
	c2nclrd

rereceive:	; rereceive (without handshake)
	txa		; backup .X
	pha
	lda c2nwrite
#if host == 16
	and #~c2nwrval
#else
	ora #c2nwrval
#endif
	sta c2nwrite	; raise WRITE
	tay
#if host == 16
	ora #c2nwrval
#else
	and #~c2nwrval
#endif
	tax
	c2nrdbit	; receive bit 0
	stx c2nwrite	; drop WRITE
	ror stemp
	c2nrdbit	; receive bit 1
	sty c2nwrite	; raise WRITE
	ror stemp
	c2nrdbit	; receive bit 2
	stx c2nwrite	; drop WRITE
	ror stemp
	c2nrdbit	; receive bit 3
	sty c2nwrite	; raise WRITE
	ror stemp
	c2nrdbit	; receive bit 4
	stx c2nwrite	; drop WRITE
	ror stemp
	c2nrdbit	; receive bit 5
	sty c2nwrite	; raise WRITE
	ror stemp
	c2nrdbit	; receive bit 6
	stx c2nwrite	; drop WRITE
	ror stemp
	c2nrdbit	; receive bit 7
	sty c2nwrite	; raise WRITE
	pla
	tax
	ldy #0
	lda stemp
	ror
#if host != 16
	eor #$ff
#endif
	rts

	; .A trashed, .Y := 00, .X preserved
send_switch:
send:
	pha
	and #$f
	jsr 0$
	pla
	lsr
	lsr
	lsr
	lsr
0$:		; send a nibble
#if host == 210 || host == 220	; 2 MHz operation
	asl		; double the pulse width
#endif
	tay
	iny
	c2nrdlo
	c2nclrd
send_nibble:
	lda #0	; placeholder
	sta c2nwrite	; drop WRITE
#if host == 16
	and #~c2nwrval
#else
	ora #c2nwrval
#endif
3$:	dey
	bne 3$
	sta c2nwrite	; raise WRITE
	rts
