;;; @file c64net.s
;;; Transfer routines for the 2-bit/4-bit PET/VIC-20/C64/C128 to PC cable
;;; used by the 64NET software
;;; @author Marko Mkel (marko.makela@nic.funet.fi)

	;; flag: is the transfer always in slow mode?
allslow = 0
	;; flag: does the receive routine leave the y register alone?
y_unmod = 0
	;; macro: additional commands for info mode
#mac infoinit
#endm
	;; macro: initialize the cable
#mac initcable
#endm
	;; macro: deinitialize the cable
#mac deinitcable
#endm
	;; macro: determine whether a request is pending
#mac isrequest
	lda data
	and #$F8	; check if the remote host wants to send a byte
	eor #8
	bne return	; jump if not

	lda #$7
	sta data
	sta ddr
#endm
	
	; .A := data, .Y := 00, .X preserved
receive_switch:
receive:
rereceive:
	lda #8
0$:
	bit data	; initial handshaking
	beq 0$
	lda data	; get the high nibble
	and #$f0
	sta stemp
	lda #8
	sta data	; acknowledge (place 0 on PB2)
1$:
	bit data	; wait for acknowledgement
	bne 1$
	ldy #4
	lda data	; get the low nibble
	sty data	; acknowledge
	ldy #0
	lsr
	lsr
	lsr
	lsr
	ora stemp
	rts

	; .A trashed, .Y := 00, .X preserved
send_switch:	; switch to sending and send
send:
	tay
0$:
	lda data	; initial handshaking
	and #$f8
	eor #$f0
	bne 0$
	lda #0
	sta data
1$:
	lda data
	bne 1$
	tya
	ora #4
	sta data	; send the lowest bit pair
	lda #8
2$:
	bit data	; wait for acknowledgement
	beq 2$
	tya
	lsr
	lsr
	tay
	and #3
	sta data	; send the lowest bit pair but one
	lda #8
3$:
	bit data	; wait for acknowledgement
	bne 3$
	tya
	lsr
	lsr
	tay
	ora #4
	sta data	; send the highest bit pair but one
	lda #8
4$:
	bit data	; wait for acknowledgement
	beq 4$
	tya
	lsr
	lsr
	sta data	; send the highest bit pair
	lda #8
5$:
	bit data	; wait for acknowledgement
	bne 5$
	lda #4
	sta data
	ldy #0
	rts
