;;; @file kontros.s
;;; Transfer routines for the 8-bit/8-bit C64/C128/Vic-20 to PC cable
;;; designed by Frank Kontros (frank@kontros.uzhgorod.ua)
;;; @author Marko Mkel (marko.makela@nic.funet.fi)

	;; flag: is the transfer always in slow mode?
allslow = 0
	;; flag: does the receive routine leave the y register alone?
y_unmod = 1

	;; macro: additional commands for info mode
#mac infoinit
#endm
	;; macro: initialize the cable
#mac initcable
#if host == 20
	lda ack
	and #$0f	; set CB1 to trigger on falling edge
	ora #$a0	; and set CB2 to pulse output handshaking
	sta ack
#endif
#if host == 64 || host == 128
	lda #<~ackval
	and ack+2
	sta ack+2	; set PA2 to input so that it won't disturb the handshaking
#endif
#endm
	;; macro: deinitialize the cable
#mac deinitcable
#endm
	;; macro: determine whether a request is pending
#mac isrequest
	lda #strval
	bit strobe	; check the -FLAG signal
	bne req$
	sta data
	jmp (oldirq)	; return
req$:
	ldx #0
	stx ddr	; restore the data lines to inputs
#endm
	; .A := data, .X and .Y preserved
receive_switch:
	lda #0	; switch back to inputs
	sta ddr
	sta data	; send the handshaking pulse

receive:
#if clearviaifr
rereceive:	; rereceive (without handshake) is only different for the CIA
#endif
	lda #strval
0$:
	bit strobe	; wait for handshaking
	beq 0$
#if clearviaifr
	sta strobe
#else
rereceive:	; rereceive (receive without handshaking) for the CIA
#endif		; clearviaifr
	lda data	; read the byte
#if clearviaifr
	sta data	; acknowledge (the VIA doesn't have read handshaking)
#endif
	rts

	; .A trashed, .Y := 00, .X preserved
send_switch:
		;;; wait for first part of handshake
	tay
	lda #strval
1$:
	bit strobe	; wait for handshaking
	beq 1$
#if clearviaifr
	sta strobe
#endif
		;;; and only then switch data direction.
	lda #$ff
	sta ddr	; set the data lines to output
	tya
	ldy #0	; clear .Y

send:
	sta data	; send the data
	lda #strval
0$:
	bit strobe	; wait for handshaking
	beq 0$
#if clearviaifr
	sta strobe
#endif
	rts
