;;; @file pc64.s
;;; Transfer routines for the 4-bit PET/C64/C128/Vic-20 to PC cable
;;; designed by Wolfgang Lorenz
;;; @author Marko Mkel (marko.makela@nic.funet.fi)

	;; flag: is the transfer always in slow mode?
allslow = 0
	;; flag: does the receive routine leave the y register alone?
y_unmod = 0

	;; macro: additional commands for info mode
#mac infoinit
#endm
	;; macro: initialize the cable
#mac initcable
#if host == 20
	lda #$ef
	and ack
	sta ack	; set CB1 to trigger on falling edge
#endif
#if host >= 201 && host <= 203
	lda ack
	and #%11111110; set CA1 to trigger on falling edge
	ora #%11100000; set CB2 to output with manual control
	sta ack
#endif
#endm
	;; macro: deinitialize the cable
#mac deinitcable
#endm
	;; macro: determine whether a request is pending
#mac isrequest
#if host == 64
	;; The Action Replay fastload command resets the PA2 line
	;; (which should be set in this protocol)
	lda #ackval
	ora ack
	sta ack
#endif
	lda #strval
	bit strobe	; check the -FLAG signal
	beq return	; jump if the client didn't want to send anything
	ldx #$f
	stx ddr	; restore the data lines to inputs/outputs
#endm

	; .A := data, .Y := 00, .X preserved
receive_switch:
receive:
#if clearviaifr
rereceive:
#endif
	lda #strval
0$:
	bit strobe	; wait for handshaking
	beq 0$
#if clearviaifr
	sta strobe
#else
rereceive:
#endif
	lda ack
	and #255 - ackval
	ldy data
	sta ack
	tya
	lsr
	lsr
	lsr
	lsr
	sta stemp
	lda #strval
1$:
	bit strobe
	beq 1$
#if clearviaifr
	sta strobe
#endif
	lda ack
	ora #ackval
	tay
	lda data
	sty ack
	and #$F0
	ora stemp
	ldy #0
	rts

	; .A trashed, .Y := 00, .X preserved
send_switch:
send:
	tay
	lda #strval
0$:
	bit strobe	; wait for handshaking
	beq 0$
#if clearviaifr
	sta strobe
#endif
	sty data
	lda ack
	and #255 - ackval
	sta ack
	tya
	lsr
	lsr
	lsr
	lsr
	tay
	lda #strval
1$:
	bit strobe	; wait for handshaking
	beq 1$
#if clearviaifr
	sta strobe
#endif
	sty data
	lda ack
	ora #ackval
	sta ack
	ldy #0
	rts
