;;; @file prlink.s
;;; Transfer routines for the 4-bit/8-bit and 8-bit/8-bit PET/C64/C128/Vic-20
;;; to Centronics cables designed by Marko Mkel and Olaf Seibert
;;; @author Marko Mkel (marko.makela@nic.funet.fi)
;;; @author Olaf Seibert (seibert@mbfys.kun.nl)

	;; flag: is the transfer always in slow mode?
allslow = 0
	;; flag: does the receive routine leave the y register alone?
y_unmod = 0

	;; macro: additional commands for info mode
#mac infoinit
#endm
	;; macro: initialize the cable
#mac initcable
#if host == 20
	lda #$ef
	and ack
	sta ack	; set CB1 to trigger on falling edge
#endif
#if host >= 201 && host <= 203
	lda ack
	and #%11111110; set CA1 to trigger on falling edge
	ora #%11100000; set CB2 to output with manual control
	sta ack
#endif
#endm
	;; macro: deinitialize the cable
#mac deinitcable
#endm
	;; macro: determine whether a request is pending
#mac isrequest
	lda #strval
	bit strobe	; check the -FLAG signal
	beq return	; jump if the client didn't want to send anything
	ldx #0
	stx ddr	; restore the data lines to inputs
#endm

	; .A := data, .Y := 00, .X preserved
receive_switch:
		;;; wait for first part of handshake
	lda #strval
1$:
	bit strobe	; wait for handshaking
	beq 1$
#if clearviaifr
	sta strobe
#endif		; clearviaifr

		;;; and only then switch data direction.
	lda #0	; switch back to inputs
	sta ddr

	beq receive1	; branch always

receive:
#if clearviaifr
rereceive:	; rereceive (without handshake) is only different for the CIA
#endif
	lda #strval
0$:
	bit strobe	; wait for handshaking
	beq 0$
#if clearviaifr
	sta strobe
#else
rereceive:	; rereceive (receive without handshaking) for the CIA
#endif		; clearviaifr
receive1:
	lda #ackval
	eor ack
	ldy data	; read the byte
#if ciadebounce
	bit strobe	; clear possible bounces
#endif
	sta ack	; acknowledge
	tya
	ldy #0
	rts

	; .A trashed, .Y := 00, .X preserved
send_switch:
		;;; wait for first part of handshake
	tay
	lda #strval
1$:
	bit strobe	; wait for handshaking
	beq 1$
#if clearviaifr
	sta strobe
#endif
		;;; and only then switch data direction.
#if prlink88
	lda #$ff
#else
	lda #$0f
#endif
	sta ddr	; set the data lines to output
	bne send1	; branch always

send:
	tay
	lda #strval
0$:
	bit strobe	; wait for handshaking
	beq 0$
#if clearviaifr
	sta strobe
#endif
send1:
#if prlink88
#else
	tya
	sta data	; send the low nibble
	lsr
	lsr
	lsr
	lsr
	tay		; move the high nibble to y
	lda #ackval
	eor ack
#if ciadebounce
	bit strobe	; clear possible bounces
#endif
	sta ack	; ack: the low nibble is on the bus
	lda #strval
1$:
	bit strobe	; handshake
	beq 1$
#if clearviaifr
	sta strobe
#endif		; via
#endif		; prlink88
	sty data	; send the high nibble (or the whole byte, for prlink88)
	ldy #0
	lda ack
	eor #ackval
#if ciadebounce
	bit strobe	; clear possible bounces
#endif
	sta ack	; ack: the high nibble is on the bus
	rts
