;;; @file x1541.s
;;; Transfer routines for the Commodore serial bus
;;; @author Marko Mkel (marko.makela@nic.funet.fi)

	;; flag: is the transfer always in slow mode?
allslow = 0
	;; flag: does the receive routine leave the y register alone?
y_unmod = 0

	;; macro: additional commands for info mode
#mac infoinit
#endm
	;; macro: initialize the cable
#mac initcable
#endm
	;; macro: deinitialize the cable
#mac deinitcable
#endm
	;; macro: determine whether a request is pending
#mac isrequest
	lda iecport1
	and #255 - clko - dato
	sta iecport1	; raise CLK and DATA.
	lda iecport2	; test if CLK and DATA are low.
	and #clki | dati
	bne return
#endm

	; .A := data, .Y := 00, .X preserved
receive_switch:
rereceive:
	lda #clki | dati
waitcd$:
	bit iecport2	; wait for CLK==DATA==low
	bne waitcd$
waitclk1$:
	lda #clko
	ora iecport1
	sta iecport1	; drop CLK
	and #255 - clko
	sta iecport1	; raise CLK
	lda iecport2
	and #clki | dati
	eor #dati
	bne waitclk1$	; repeat until CLK==low && DATA==high

	lda #dato
	ora iecport1
	sta iecport1	; drop DATA
	lda #clki
waitclk2$:
	bit iecport2
	beq waitclk2$	; wait for CLK==high

	lda #255 - dato
	and iecport1
	sta iecport1	; raise DATA

receive:
	ldy #8	; get 8 bits
loop$:
	lda iecport2
	and #dati | clki
	eor #dati | clki
	beq loop$	; wait for CLK==low || DATA==low
getrest$:
#if dati == 128
	asl		; Carry = DATA==low
#else
#if dati < clki
	and #dati
#endif
	cmp #dati
#endif

	lda iecport1
	and #255 - dato - clko
	eor #clko
	bcs skip$
	eor #dato | clko
skip$:
	sta iecport1	; acknowledge the bit
	ror stemp	; store the data

	lda #dati | clki
getack$:
	bit iecport2
	beq getack$	; wait for CLK==high || DATA==high

	lda iecport1
	and #255 - clko - dato
	sta iecport1	; raise CLK and DATA
	dey
	bne loop$	; loop until all bits are sent
	lda stemp
	rts

	; .A trashed, .Y := 00, .X preserved
send_switch:
	tay
waitclk$:
	lda iecport1
	ora #clko | dato
	sta iecport1	; drop CLK and DATA
	and #255 - clko - dato
	sta iecport1	; raise CLK and DATA
	lda iecport2
	and #clki	; repeat until CLK==low
	bne waitclk$

	lda iecport1
	ora #clko
	sta iecport1	; drop CLK
	lda #dati
waitdata1$:
	bit iecport2
	bne waitdata1$; wait for DATA==low

	lda iecport1
	and #255 - clko
	sta iecport1	; raise CLK
	lda #dati
waitdata2$:
	bit iecport2	; wait for DATA==high
	beq waitdata2$
	tya

send:
	sta stemp
	ldy #8	; send 8 bits
loop$:
	lsr stemp	; read a bit
	lda iecport1
	and #255 - dato - clko
	eor #clko
	bcc skip$
	eor #dato | clko
skip$:
	sta iecport1	; send the data

	lda #dati | clki
getack1$:
	bit iecport2
	bne getack1$	; wait for CLK==DATA==low

	lda #255 - clko - dato
	and iecport1
	sta iecport1	; set DATA=CLK=high
wputack2$:
	lda iecport2
	and #dati | clki
	eor #dati | clki
	bne wputack2$	; wait for CLK==DATA==high
	dey
	bne loop$	; loop until all bits are sent
	rts
