;;; @file c64common.s
;;; Common definitions for Commodore 64 like computers (c64 and c128)
;;; @author Marko Mkel (marko.makela@nic.funet.fi)

;;; in the 6522, the interrupt flag has to be reset
clearviaifr = 0
;;; in the 6526, the FLAG line may need debouncing in the C64/C128
ciadebounce = 1

;;; Commodore serial bus definitions
iecport1 = $dd00
iecport2 = iecport1
atno = 8
clko = 16
dato = 32
clki = 64
dati = 128
;;; Commodore cassette port definitions
c2nread = $dc0d
c2nrdval = 16
c2nwrite = 1
c2nwrval = 8

;;; miscellanous
strval	= $10	; strobe bit value

;;; 6551 ACIA definitions (for a 6551 hooked at -I/O1)
acia = $de00
r6551dt = acia			; transmit/receive data
r6551sr = acia + 1		; programmed reset/status register
r6551cm = acia + 2		; command register
r6551ct = acia + 3		; control register

;;; memory locations
stemp	= $a7	; memory place that may hold the byte being received

start	= $ac	; start address of the transfer area
end	= $ae	; end address of the transfer
cinv	= $314	; interrupt vector

ackval	= 4
ack	= $dd00
data	= $dd01
ddr	= $dd03
strobe	= $dd0d

	;; program header
#mac header
#endm

	;; store the speed settings and enable fast speed
#mac phfast
	lda $d011
	pha
	lda $d030
	pha
	lda #$1
	sta $d011	; blank the screen
	sta $d030	; switch to 2 MHz mode
#endm
	;; restore original speed from fast speed
#mac plfast
	pla
	sta $d030	; restore original speed
	pla
	sta $d011	; and screen mode
#endm
	;; store the speed settings and enable slow speed (no badlines)
#mac phslow
	lda $d011
	pha
	lda $d015
	pha
	lda $d030
	pha
	lda #$0
	sta $d015	; disable the sprites
	sta $d011	; blank the screen
	sta $d030	; switch to 1 MHz mode
nobad$:	bit $d011	; wait for vertical blank area
	bpl nobad$
#endm
	;; restore original speed from slow speed
#mac plslow
	pla
	sta $d030	; restore the speed
	pla
	sta $d015	; restore the sprite configuration
	pla
	sta $d011	; restore the screen mode
#endm
