;;; @file nonebase.s
;;; Basic definitions for no memory expansion
;;; @author Marko Mkel (marko.makela@nic.funet.fi)

	;; increment and compare pointers
#mac plain_incmptr_decl
incmptr:	; increment and compare the current byte pointer.
	inc start	; increment.
	bne 0$
	inc start+1
0$:	lda start	; compare.  if start == end, the zero flag will be set.
	cmp end
	bne 1$
	lda start+1
	cmp end+1
1$:	rts
#endm
	;; load a memory block
#mac plain_load
#if y_unmod
	ldy #0	; The receive does not clear the .Y register
#endif
#if host == 128
	lda pokeptr
	pha
	lda #start
	sta pokeptr

2$:	jsr receive	; get the bytes
	ldx bank
	jsr poke
	jsr incmptr	; update the pointer
	bne 2$

	sty bank
	pla
	sta pokeptr
#else		; host == 128
2$:	jsr receive	; get the bytes
	sta (start),y
	jsr incmptr	; update the pointer
	bne 2$
#endif		; host == 128
#endm

	;; save a memory block
#mac plain_save
#if host == 128
	lda peekptr
	pha
	lda #start
	sta peekptr

#if send != send_switch
	ldx bank	; unrolled once
	jsr peek
	jsr send_switch
	jsr incmptr
	beq skiploop$
#endif		; send != send_switch

loop$:	ldx bank
	jsr peek
	jsr send
	jsr incmptr
	bne loop$

skiploop$:
	sty bank
	pla
	sta peekptr
#else		; host == 128
#if y_unmod
	ldy #0	; The receive does not clear the .Y register
#endif
#if host == 64
#if send != send_switch
	ldx 1		; unrolled once
	lda #$3c
	sta 1		; switch to 64 kB RAM config
	lda (start),y
	stx 1		; restore the normal config
	jsr send_switch
	jsr incmptr
	beq skiploop$
#endif		; send != send_switch

	ldx 1
loop$:	lda #$3c
	sta 1		; switch to 64 kB RAM config
	lda (start),y
	stx 1		; restore the normal config
#else		; host == 64
#if send != send_switch
	lda (start),y	; unrolled once
	jsr send_switch
	jsr incmptr
	beq skiploop$
#endif		; send != send_switch
loop$:	lda (start),y
#endif		; host == 64
	jsr send
	jsr incmptr
	bne loop$
skiploop$:
#endif		; host == 128
#endm
