;;; @file piabase.s
;;; Definitions for the MC 6821 PIA controlled memory expansion
;;; for the Commodore 64 and 128
;;; @author Marko Mkel (marko.makela@nic.funet.fi)

	;; flag: is there a RAM expansion
ramexp	= 1
	;; the PIA base address
pia = $d7c0
	;; initialize the PIA
#mac piainit
#if host == 128
	lda #$3e
	sta $ff00
#endif
	ldx #11	; initialize the PIA
0$:
	lda piatable-1,x
	sta pia,x
	dex
	bne 0$
#endm

	;; initialization data
#mac piadata
piatable:
	.byte $34,$fe,4,$ff,0,$ff,0,$dc,4,$fe,4
piainit:
#if host == 128
	;; .A must contain the $d506 contents when calling this routine
	ora #$3f
	sta $d506	; select 32kB common memory (top and low)
	lda bank
	and #$f
	lsr
	ror
	ror
	tax
	and #$c0
	ora #2
	sta $d500	; set the MMU banking (disabling the BASIC ROM)
	txa
	ror
	ror
#else
	lda bank
	asl
	asl
	asl
	asl
#endif
	eor #$c0
	and #$c0
	ora #$c
	sta pia	; store the PIA banking configuration
	lda start+1
	lsr
	lsr
	and #$30
	ora pia
	sta pia	; set the initial PIA configuration
#endm

	;; increment and compare pointers
#mac pia_incmptr_decl
incmptr:		; increment and compare the current byte pointer
	inc start	; and change the PIA bank if necessary.
	bne 0$
	inc start+1
	bpl 0$
	dex		; decrease the bank jump counter
	lda #$40
	sta start+1	; restore the address
	clc
	lda #$10
	adc pia
	sta pia		; and adjust the PIA bank
0$:	txa		; is it the last bank?
	bne 1$		; no, return
	lda start	; perform the comparison
	cmp end
	bne 1$
	lda start+1
	cmp end+1
1$:	rts
#endm

	;; load a memory block
#mac plain_load
	lda end+1	; store the end of file pointer high byte
	pha
	lda pia	; store the PIA memory configuration
	pha
#if host == 128
	lda $d506	; store the common memory configuration
	pha
#endif
	jsr piainit	; initialize the PIA expansion (with $d506 value in A)
#if y_unmod
	ldy #0	; The receive does not clear the .Y register
#endif
2$:	jsr receive
	sta (start),y
	jsr incmptr
	bne 2$

#if host == 128
	pla		; restore the original MMU banking
	sty $d500	; enable all ROMs, select RAM bank 0
	sta $d506	; restore the common memory configuration
#endif
	pla
	sta pia
	pla
	sta start+1	; adjust the end address pointers
	sta end+1
#endm

	;; save a memory block
#mac plain_save
	lda pia	; store the PIA memory configuration
	pha
#if host == 128
	lda $d506	; store the common memory configuration
	pha
#endif
	jsr piainit	; initialize the PIA expansion (with $d506 value in A)
#if y_unmod
	ldy #0	; The receive does not clear the .Y register
#endif
#if send != send_switch
	lda (start),y	; unrolled once
	jsr send_switch
	jsr incmptr
	beq skiploop$
#endif		; send != send_switch
loop$:
	lda (start),y
	jsr send
	jsr incmptr
	bne loop$
skiploop$:

#if host == 128
	pla
	sty $d500
	sta $d506
#endif
	pla
	sta pia
#endm
