;;; @file reubase.s
;;; Definitions for the Commodore RAM Expansion Unit
;;; for the Commodore 64 and 128
;;; @author Marko Mkel (marko.makela@nic.funet.fi)

	;; flag: use Action Replay and the REU simultaneously
actionreuplay = 0

	;; smallest REU bank number
reubmin = 16
	;; base address of the REU
reu = $df00

	;; flag: is there a RAM expansion
ramexp	= 1
	;; auxiliary routines
#mac reu_aux
reuinit:
	lda #0
	ldx #$40
	sta reu+2	; set the buffer address for the C64/C128
	stx reu+3
	sta reu+7	; set the buffer length
	stx reu+8
	sta reu+9	; reset some other registers
	sta reu+10
	lda bank
	sbc #reubmin	; set the memory bank (C flag initially set)
	sta reu+6
	lda start	; set the start address in the REU
	sta reu+4
	lda start+1
	sta reu+5
#if actionreuplay
	sta bank
#endif
#if host == 128
	lda #2
	sta $ff00	; switch to RAM bank 0 and disable the BASIC ROM
#endif
#if !allslow
	dec $d030	; slow down to 1 MHz
#endif
	lda #$f2
	sta reu+1	; swap the buffers
#if !allslow
	inc $d030	; speed up again
#endif
	jmp rptrinit

incmptreu:
	inc start	; increment and compare the current byte pointer
	bne 0$		; and transfer another REU block in if necessary.
	inc start+1
	bpl 0$
	dex		; decrease the bank jump counter
#if !allslow
	dec $d030	; slow down to 1 MHz
#endif
	lda #$f2
	sta reu+1	; swap the buffers
	lda #$40
	sta start+1	; restore the REU buffer address
	clc
#if actionreuplay
	adc bank
	sta bank
#else
	adc reu+5
#endif
	sta reu+5	; update the REU memory pointer
	lda #$f2
	sta reu+1	; swap the buffers
#if !allslow
	inc $d030	; switch back to 2 MHz mode
#endif
0$:	txa		; is it the last bank?
	bne 1$		; no, return
	lda start	; perform the comparison
	cmp end
	bne 1$
	lda start+1
	cmp end+1
1$:	rts
#endm	
	;; load a memory block
#mac reu_load
	lda bank
	cmp #reubmin	; is it a REU bank?
	bcc noreuld	; no, do not save REC registers on stack

#if actionreuplay
	lda #$2a
	sta $de00	; enable Action Replay RAM at $df00 (disable the ROM)
		; Action Replay has one write-only I/O register at $de00-$deff.
		; Its bits are as follows:
		; 7 - unused, or extra ROM bank selector (A15)
		; 6 - Resets FREEZE-mode (turns back to normal mode)
		; 5 - 1=enable RAM at ROML ($8000) & I/O2 ($DF00, =$9F00-$9FFF)
		; 4 - ROM bank selector high (A14)
		; 3 - ROM bank selector low (A13)
		; 2 - 1=disable cartridge (turn off the $de00 register)
		; 1 - 1=-EXROM high
		; 0 - 1=-GAME low
#endif
	lda end+1	; store the end of file pointer high byte
	pha
	jsr reuinit	; initialize the REU registers.	 C flag is set at this stage.
#if y_unmod
	ldy #0	; The receive does not clear the .Y register
#endif
12$:
	jsr receive
	sta (start),y
	jsr incmptreu
	bne 12$

#if !allslow
	dec $d030	; switch to 1 MHz mode
#endif
	lda #$f2
	sta reu+1	; swap the REU memory back
	pla
	sta start+1	; adjust the end address pointers
	sta end+1

#if actionreuplay
	lda #$0a
	sta $de00	; restore normal Action Replay configuration
#endif		; actionreuplay
	jmp endload

noreuld:
	plain_load
#endm

	;; save a memory block
#mac reu_save
	lda bank
	cmp #reubmin	; is it a REU bank?
	bcc noreusv	; no, jump to normal save

#if actionreuplay
	lda #$2a
	sta $de00	; disable Action ROM at $df00
#endif
	lda end+1	; store the end of file pointer high byte
	pha
	jsr reuinit	; initialize the REU registers.	 C flag is set at this stage.
#if y_unmod
	ldy #0	; The receive does not clear the .Y register
#endif
#if send != send_switch
	lda (start),y	; unrolled once
	jsr send_switch
	jsr incmptreu
	beq skiploop$
#endif		; send != send_switch
loop$:
	lda (start),y
	jsr send
	jsr incmptreu
	bne loop$
skiploop$:

#if !allslow
	dec $d030	; switch to 1 MHz mode
#endif
	lda #$f2
	sta reu+1	; swap the REU memory back
	pla
	sta start+1
	sta end+1

#if actionreuplay
	lda #$0a
	sta $de00	; restore normal Action Replay configuration
#endif		; actionreuplay
#if host == 128
	sty $ff00	; restore normal MMU configuration
;;; TO DO: restore normal MMU configuration (no common memory?)
#endif
	jmp endsave

noreusv:
	plain_save
#endm
