/**
 * @file amigapar.c
 * Interface to accessing the Amiga parallel port
 * @author Marko Mkel (marko.makela@nic.funet.fi)
 * @author Olaf Seibert (rhialto@mbfys.kun.nl)
 */

/*
 * Copyright  1994-1997 Marko Mkel and Olaf Seibert
 * Copyright  2001 Marko Mkel
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#if defined amiga || defined AMIGA || defined __amiga__ || defined __AMIGA__
# include "amigapar.h"
# include <exec/exec.h>
# include <resources/misc.h>
# include <clib/exec_protos.h>
# include <clib/misc_protos.h>

# include <stdio.h>

/** base address of the miscellanous library */
static struct Library* MiscBase = 0;
/** flag: which parts of the parallel port have been allocated? */
static unsigned long allocated = 0;

/** Reserve the parallel port
 * @param ctl	flag: reserve the control lines as well
 * @return	0 on success, nonzero on failure
 */
int
alloc_port (int ctl)
{
  /** name of the reserving application */
  static const char myname[] = "cbmlink";

  if (!(MiscBase = OpenResource ("misc.resource"))) {
    fputs ("OpenResource: cannot open misc.resource\n", stderr);
    return 1;
  }

  if (allocated & 1 << MR_PARALLELPORT);
  else {
    register const char* user = AllocMiscResource (MR_PARALLELPORT, myname);
    if (!user) {
      fputs ("Printer data lines are used by ", stderr);
      fputs (user, stderr);
      fputs (".\n", stderr);
      return 2;
    }
    allocated |= 1 << MR_PARALLELPORT;
  }
  if (allocated & 1 << MR_PARALLELBITS);
  else {
    register const char* user = AllocMiscResource (MR_PARALLELBITS, myname);
    if (!user) {
      fputs ("Printer control lines are used by ", stderr);
      fputs (user, stderr);
      fputs (".\n", stderr);
      return 2;
    }
    allocated |= 1 << MR_PARALLELBITS;
  }

  return 0;
}

/** Give back the parallel port
 * @return	0 on success, nonzero on failure
 */
int
free_port (void)
{
  if (allocated & 1 << MR_PARALLELBITS) {
    /* turn POUT back into input */
    CTL_DDR &= ~POUT;
    FreeMiscResource (MR_PARALLELBITS);
  }
  if (allocated & 1 << MR_PARALLELPORT) {
    /* set all data lines to output high */
    DATA = 0xFF;
    DATA_DDR = 0xFF;
    FreeMiscResource (MR_PARALLELPORT);
  }
  allocated = 0;
}

#endif /* amiga || AMIGA || __amiga__ || __AMIGA__ */
