/**
 * @file pcpar.c
 * Interface to accessing IBM PC like parallel ports
 * @author Marko Mkel (marko.makela@nic.funet.fi)
 */

/*
 * Copyright  2001 Marko Mkel
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define _BSD_SOURCE
#ifdef COMM_PC
# include "pcpar.h"
# include <stdio.h>
# include <stdlib.h>

#ifdef __BCC__
/* These are from bcc_io.c */
# asm
! int inportb (int port);
	.globl	_inportb
_inportb:
	pop	bx
	pop	dx
	dec	sp
	dec	sp
	in	al,dx
	sub	ah,ah
	jmp	bx

! void oportb (int port, char value);
	.globl	_oportb
_oportb:
	pop	bx
	pop	dx
	pop	ax
	sub	sp,*4
	out	dx,al
	jmp	bx
# endasm
#endif /* __BCC__ */

/** the base address for the printer port, and the address of the data port */
unsigned baseaddr;
/** the address of the status port */
unsigned stataddr;
/** the address of the control port */
unsigned ctrladdr;

/** recognized base addresses for ports */
static unsigned portbase[] = { 0x3bc, 0x378, 0x2bc, 0x278 };

/** Open a parallel port
 * @param dev	the "device name" of the port
 * @return	the base address of the port, or 0 on failure
 */
unsigned
open_port (const char* dev)
{
  char* s;
  unsigned long p = strtoul (dev, &s, 0);
  if (!*dev || *s) {
    fprintf (stderr, "open_port: unrecognized port %s\n", dev);
    return 0;
  }
  if (p < (sizeof portbase) / sizeof *portbase)
    baseaddr = portbase[p];
  else {
    unsigned i = (sizeof portbase) / sizeof *portbase;
    while (i)
      if (p == portbase[--i])
	goto found;
    fprintf (stderr, "open_port: port %s out of range\n", dev);
    return 0;
  found:
    baseaddr = portbase[i];
  }

# ifdef __linux__
  if (ioperm (baseaddr, 3, 1)) {
    perror ("open_port: ioperm");
    return 0;
  }

  /* give up the super user privileges */
  seteuid (getuid ());
# endif /* __linux__ */
# ifdef __FreeBSD__
  /* TO DO: gain access to the ports */

  /* give up the super user privileges */
  seteuid (getuid ());
# endif /* __FreeBSD__ */

  stataddr = baseaddr + 1;
  ctrladdr = baseaddr + 2;
  return baseaddr;
}
#endif /* COMM_PC */
