/**
 * @file pcpar.h
 * Interface to accessing IBM PC like parallel ports
 * @author Marko Mkel (marko.makela@nic.funet.fi)
 */

/*
 * Copyright  2001 Marko Mkel
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef COMM_PC
# ifdef __linux__
#  include <unistd.h>
#  include <sys/io.h>
# endif /* __linux */
# ifdef __FreeBSD__
#  include <unistd.h>
#  include <machine/cpufunc.h>
# endif /* __FreeBSD__ */
# if defined __WIN32__ && defined __GNUC__
/* copied from Linux <sys/io.h> */
static __inline unsigned char
inb (unsigned short int port)
{
  unsigned char _v;
  __asm__ __volatile__ ("inb %w1,%0":"=a" (_v):"Nd" (port));
  return _v;
}
static __inline void
outb (unsigned char value, unsigned short int port)
{
  __asm__ __volatile__ ("outb %b0,%w1": :"a" (value), "Nd" (port));
}
# endif /* __WIN32__ */
# ifdef __BCC__
#  define outb(value,port) oportb(port, value)
#  define inb(port) inportb(port)
# endif /* __BCC__ */
#endif /* COMM_PC */

#ifdef __GNUC__
# define ASM(label) __asm__ (#label)
#else /* __GNUC__ */
# define ASM(label)
#endif /* __GNUC__ */

/** the base address for the printer port, and the address of the data port */
extern unsigned baseaddr ASM(baseaddr);
/** the address of the status port */
extern unsigned stataddr ASM(stataddr);
/** the address of the control port */
extern unsigned ctrladdr ASM(ctrladdr);

/** Open a parallel port
 * @param dev	the "device name" of the port
 * @return	the base address of the port, or 0 on failure
 */
unsigned
open_port (const char* dev);
